#include "stdafx.h"

class CGenRate : public CItem { 
	public : 
		double m_dPerturb;		// amount to 'perturb' this rate ( todo - split to fwd and bwd portions )

		double m_dBaseRate[2];	// unperturbed rate, fwd and bwd
		double m_dPertRate[2];	// perturbed rate 
		CGenRate() { 
			m_dPerturb=0.0;
			m_dBaseRate[0]=m_dBaseRate[1]=1.0;
			m_dPertRate[0]=m_dPertRate[1]=1.0;
			}
		double Rate( bool bBase, int iFwdBwd ) {		// iFwdBwd=0 fwd, 1 bwd 
			return bBase ? m_dBaseRate[iFwdBwd] : m_dPertRate[iFwdBwd];
			}
	};

class CGen { 
	public : 
				
		int m_iStates;
		double m_dHeight;
		double m_dDiffusion;
		double m_dKRef;
		
		CItemArray m_aRates;
		double Rate( int iRate, bool bBase, int iFwdBwd ) { 
			return ((CGenRate  *)m_aRates[iRate])->Rate( bBase, iFwdBwd );
			}
		
		//-- fill in values ( from cosntructor or control entries )
		void SetValues( int iStates, double dHeight, double dDiffusion, double dKRef ) {
			m_iStates=iStates;
			m_dHeight=dHeight;
			m_dDiffusion=dDiffusion;
			m_dKRef=dKRef;
			}
			
		//-- default constructor
		CGen() { 
			SetValues( 12, 1.0, 0.1, 100000 );
			}
			
		//-- fill in m_aRates based on settings : m_iStates
		void Calculate() { 
			double dx = double( m_iStates - 1 ) / 2.0;					
			double dx2 = dx * dx ;
			double dPrevY = 0.0;
			double dLnK0 = log( m_dKRef );

			for( int iState=1; iState<m_iStates; ++iState ) { 
				while( iState-1 >= m_aRates.Count() )
					m_aRates.AddItem( new CGenRate );
				
				CGenRate * pRate = (CGenRate *)(m_aRates[iState-1]);
				
				double xr = double(iState) - 0.5 - dx ;			// psuedo x value for rate 
				double yr = m_dHeight - m_dHeight * xr * xr / dx2 + m_dDiffusion;	// height for rate

				pRate->m_dBaseRate[0] = exp(dPrevY-yr+dLnK0);
				pRate->m_dPertRate[0] = exp(dPrevY-yr+dLnK0-pRate->m_dPerturb/2.0);

				// calc x and y values for state calculation [parabola]
				// (eventually discarded - not needed to build model )
				double x = double(iState)-dx;						// pseudo x value for state 
				double y = m_dHeight - m_dHeight * x * x / dx2;		// height for state 
				
				// phi value  = kb / kf 
				pRate->m_dBaseRate[1] = exp( y-dPrevY ) * pRate->m_dBaseRate[0] ;
				pRate->m_dPertRate[1] = exp( y-dPrevY+pRate->m_dPerturb ) * pRate->m_dPertRate[0] ;
			
				dPrevY = y;
				}
			}

		//-- Generate the model and open in model editor dialogue
		void Generate( bool bBase ) { 
			CModel oModel;
			oModel.m_aClasses.AddItem( new CModelClass( 0.0, 0.1, 1.0, DefaultClassColor(0) ) );
			oModel.m_aClasses.AddItem( new CModelClass( 1.0, 0.15, 1.0, DefaultClassColor(1) ) );
			oModel.AddState( 0, -90.0, 0.0, 1.0 );

			double dTerm=1.0, dRi, dNX = 0.0;
			char bufNX[256];				// sprintf message buffer

			for( int iState=1; iState<m_iStates; ++iState ) { 
				CGenRate * pRate = (CGenRate *)(m_aRates[iState-1]);
				oModel.AddState( iState==m_iStates-1?1:0, double(iState)*180.0/(m_iStates-1)-90, 0.0, 0.0 );
				oModel.m_aRates.AddItem( 
						new CModelRate( iState-1, iState, pRate->Rate(bBase,0), pRate->Rate(bBase,1) ) 
						);

				// NX = 1 + r1 + r1r2 + r1r2r3 + r1r2r3r4 ... 
				if( iState>1 ) { 
					dRi = Rate(iState-2, bBase, 1) / Rate(iState-1,bBase,0);	// previous kb / current kf 
					dTerm *= dRi;

					sprintf( bufNX, "  %d r=%f     product=%f   ( %f/%f )", iState-1, dRi, dTerm, Rate(iState-2, bBase, 1), Rate(iState-1,bBase,0) );
					AddMessage( REPTYPE_NOTE, bufNX );
					}
				dNX += dTerm;


				}

			sprintf( bufNX, "nx=%f  1/nx=%f", dNX, 1.0/dNX );
			AddMessage( REPTYPE_NOTE, bufNX );

			SendMessage( OpenModelView( "Generated Model" ), WM_MODEL_SETMODEL, 0, (LPARAM)&oModel );
			}		
			
	};
	
CGen g_oGen;

//---------------------------------------------------------------------------------
//----- Set dialog control settings based on g_oGen 
void Setup( HWND hDlg ) { 
	//-- States and Height 
	char buf[64];
	sprintf( buf, "%d", g_oGen.m_iStates );
	SetWindowText( GetDlgItem( hDlg, IDC_EDTSTATES ),  buf );			
	
	sprintf( buf, "%f", g_oGen.m_dHeight );
	SetWindowText( GetDlgItem( hDlg, IDC_EDTHEIGHT ),  buf );			
	
	sprintf( buf, "%f", g_oGen.m_dDiffusion );
	SetWindowText( GetDlgItem( hDlg, IDC_EDTDIFFCONST ),  buf );			
	
	sprintf( buf, "%f", g_oGen.m_dKRef );
	SetWindowText( GetDlgItem( hDlg, IDC_EDTREFERENCEK ), buf );
	}
	
//----- Copy rates to list control 
void SetupList( HWND hDlg ) { 
	HWND hLst = GetDlgItem( hDlg, IDC_LSTRATES );
	SendMessage( hLst, LB_RESETCONTENT, 0, 0 );
	char buf[256];
	for( int iRate=0; iRate<g_oGen.m_aRates.Count(); ++iRate ) { 
		CGenRate * pRate = (CGenRate *)(g_oGen.m_aRates[iRate]);
		sprintf( buf, "[%d-%d] Perturb:%.2f  [%.2f,%.2f]", iRate, iRate+1, pRate->m_dPerturb, pRate->m_dPertRate[0], pRate->m_dPertRate[1] );
		SendMessage( hLst, LB_ADDSTRING, 0, (LPARAM) (LPCTSTR) buf );
		}
	}
	
//----- Extract dialogue control settings and place in g_oGen
void Teardown( HWND hDlg ) { 
	char buf[256];
	GetWindowText( GetDlgItem( hDlg, IDC_EDTSTATES ), buf, 256 );
	g_oGen.m_iStates = atoi( buf );

	GetWindowText( GetDlgItem( hDlg, IDC_EDTHEIGHT ), buf, 256 );
	g_oGen.m_dHeight = atof( buf );

	GetWindowText( GetDlgItem( hDlg, IDC_EDTDIFFCONST ), buf, 256 );
	g_oGen.m_dDiffusion = atof( buf );

	GetWindowText( GetDlgItem( hDlg, IDC_EDTREFERENCEK ), buf, 256 );
	g_oGen.m_dKRef = atoi( buf );
	}
	
//---------------------------------------------------------------------------------
LRESULT CALLBACK DlgModelGenerator( HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam ){
	switch (message){
		case WM_INITDIALOG: 
			Setup( hDlg );
			return TRUE;

		case WM_COMMAND: {
			int wmId = LOWORD(wParam); 
			
			switch( wmId ) { 
				case IDC_BTNGENRATES :
					Teardown( hDlg );
					g_oGen.m_aRates.DeleteAll();
					g_oGen.Calculate();
					SetupList( hDlg );
					break;

				case IDC_BTNCHANGELISTITEM : {
					int iRow, iRows;
					HWND hLst = GetDlgItem( hDlg, IDC_LSTRATES );
					if( ( iRows = (int) SendMessage( hLst, LB_GETCOUNT, 0, 0 ) )  > 0 
							&& ( iRow = (int) SendMessage( hLst, LB_GETCURSEL, 0, 0 ) ) >= 0 
							&& iRow<g_oGen.m_aRates.Count() 
							) { 
						CGenRate  * pRate = ((CGenRate  *)g_oGen.m_aRates[iRow]);
						char buf[256];
						GetWindowText( GetDlgItem( hDlg, IDC_EDTPHI ), buf, 256 );
						pRate->m_dPerturb = atof( buf );
						
						g_oGen.Calculate();
						SetupList( hDlg );
						}
					}
					break;

				case IDC_BTNGENMODEL :
				case IDC_BTNGENPERTMODEL :
					if( g_oGen.m_aRates.Count()==0 ) 
						DlgModelGenerator( hDlg, WM_COMMAND, IDC_BTNGENRATES, 0 );
					g_oGen.Generate( wmId == IDC_BTNGENMODEL );
					break;
										
				case IDCANCEL : 
					EndDialog(hDlg, LOWORD(wParam));
					return TRUE;
					break;
				}
			break;
			}
			
		}
		
	return FALSE;
	}

bool OpenModelGenerator( HWND hWndOwner ) { 
	return IDOK == DialogBox( GetInstance( ), (LPCTSTR)IDD_DLG_MODEL_GENERATOR, hWndOwner, (DLGPROC)DlgModelGenerator) ;
	}


